#!/bin/bash

#        Clone gists from GitHub to Opengist
#        Copyright (C) 2024 Jarno Rankinen
#
#        This program is free software: you can redistribute it and/or modify
#        it under the terms of the GNU Affero General Public License as
#        published by the Free Software Foundation, either version 3 of the
#        License, or (at your option) any later version.
#
#        This program is distributed in the hope that it will be useful,
#        but WITHOUT ANY WARRANTY; without even the implied warranty of
#        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#        GNU Affero General Public License for more details.
#
#        You should have received a copy of the GNU Affero General Public License
#        along with this program.  If not, see <https://www.gnu.org/licenses/>.


# Requirements: jq, gh-cli (gh), git, sqlite3

#---------------------------------------------------
# Github access details, give token the "gist" scope
#---------------------------------------------------
TOKEN=''
GITHUB_USER=''
PER_PAGE=100

#-----------------------------------------------------------------------------------------
# Opengist access details:
#
# OG_URL is the instance url without scheme, e.g 'opengist.example.com'
# Set a password for the Opengist user and put username and password into OG_USER and OG_PW
# SQLITE_DB is the path to the sqlite db of Opengist, e.g. '/path/to/opengist/opengist.db'
#-----------------------------------------------------------------------------------------
OG_URL=''
OG_USER=''
OG_PW=''
SQLITE_DB=''

### Check requirements
requirements=( jq gh git sqlite3 )
for req in ${requirements[@]}; do
	if !command -v $req &> /dev/null; then
		echo "This script needs $req installed"
		exit 1
	fi
done

#--------------------------
# Get gist info from GitHub
#--------------------------
data="$(curl -sL \
	-H "Accept: application/vnd.github+json" \
	-H "Authorization: Bearer $TOKEN" \
	-H "X-GitHub-Api-Version: 2022-11-28" \
	https://api.github.com/users/$GITHUB_USER/gists?per_page=$PER_PAGE)"
tmp=$(mktemp)
echo $data > $tmp

#-------------------------------
# Generate ids and descrs arrays
#-------------------------------
ids=($(jq -r '.[] | .id' $tmp))
IFS=$'\n' read -rd '' -a descrs <<<$(echo $data | jq ".[] | .description" | sed "s/\"/\'/g")

#--------------------------------
# For each gist in GitHub,
# - clone the gist
# - init new gist in Opengist
# - Alter the OG Database and set:
#   - Title as the filename
#   - Access level like on github
#     ( unlisted -> private )
#   - Description
#---------------------------------
i=0
for id in ${ids[@]}; do
    ### Number of files
	l=$(jq -r ".[] | select(.id==\"$id\") | .files | length" "$tmp")
    ### Visibility
	public=$(jq -r ".[] | select(.id==\"$id\") | .public" "$tmp")
	private=2
	[[ "$public" == "true" ]] && private=0
    ### Name of first file
	f0=$(jq -r ".[] |  select(.id==\"$id\") | .files | keys_unsorted[0]" "$tmp")
    ### Description
	desc=${descrs[$i]}
	[[ "${descrs[$i]}" == "''" ]] && desc=\'$f0\'
	echo -e "$i: ${id@Q} ${descrs[$i]} files:$l first:${f0@Q} private:${private@Q}"
    ### Clone gist
	gh gist clone $id $id
	pushd "$id" &> /dev/null || exit 1
    ### Push as new gist to Opengist, get the UUID of the new gist
	git remote add gist https://$OG_USER:$OG_PW@$OG_URL/init
    uuid=$(git push -u gist --all 2>&1 | grep set-url | awk -F "/" '{print $NF}' | sed 's|\s||g')
    ### Set title, visibility and description
	sqlite3 "$SQLITE_DB" "update gists set title = ${f0@Q}, private = ${private@Q}, description = ${desc} where uuid = ${uuid@Q}"
	printf "Created gist %s " ${uuid@Q}
    popd &> /dev/null || exit 1
	rm -rf "$id"
	((i++))
done

echo $tmp
